package net.jhorstmann.jsonp.xmlutil;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.Reader;
import javax.xml.XMLConstants;
import org.xml.sax.Attributes;
import org.xml.sax.ContentHandler;
import org.xml.sax.DTDHandler;
import org.xml.sax.EntityResolver;
import org.xml.sax.ErrorHandler;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;
import org.xml.sax.SAXNotRecognizedException;
import org.xml.sax.SAXNotSupportedException;
import org.xml.sax.XMLReader;
import org.xml.sax.helpers.AttributesImpl;

public abstract class AbstractXMLReader implements XMLReader {

    protected static final String DEFAULT_ENCODING = "utf-8";
    protected static final Attributes EMPTY_ATTRIBUTES = new AttributesImpl();
    protected ContentHandler contentHandler;
    protected ErrorHandler errorHandler;
    protected DTDHandler dtdHandler;
    protected EntityResolver entityResolver;

    public boolean getFeature(String name) throws SAXNotRecognizedException, SAXNotSupportedException {
        if ("http://xml.org/sax/features/namespaces".equals(name)) {
            return true;
        } else if ("http://xml.org/sax/features/namespace-prefixes".equals(name)) {
            return false;
        } else {
            throw new SAXNotRecognizedException();
        }
    }

    public void setFeature(String name, boolean value) throws SAXNotRecognizedException, SAXNotSupportedException {
        if ("http://xml.org/sax/features/namespaces".equals(name)) {
            if (!value) {
                throw new SAXNotSupportedException();
            }
        } else if ("http://xml.org/sax/features/namespace-prefixes".equals(name)) {
            if (value) {
                throw new SAXNotSupportedException();
            }
        } else {
            throw new SAXNotRecognizedException();
        }
    }

    public Object getProperty(String name) throws SAXNotRecognizedException, SAXNotSupportedException {
        throw new SAXNotRecognizedException();
    }

    public void setProperty(String name, Object value) throws SAXNotRecognizedException, SAXNotSupportedException {
        throw new SAXNotRecognizedException();
    }

    public DTDHandler getDTDHandler() {
        return dtdHandler;
    }

    public void setDTDHandler(DTDHandler dtdHandler) {
        this.dtdHandler = dtdHandler;
    }

    public EntityResolver getEntityResolver() {
        return entityResolver;
    }

    public void setEntityResolver(EntityResolver entityResolver) {
        this.entityResolver = entityResolver;
    }

    public void setContentHandler(ContentHandler handler) {
        this.contentHandler = handler;
    }

    public ContentHandler getContentHandler() {
        return contentHandler;
    }

    public void setErrorHandler(ErrorHandler handler) {
        this.errorHandler = handler;
    }

    public ErrorHandler getErrorHandler() {
        return errorHandler;
    }

    public void parse(InputSource input) throws IOException, SAXException {
        Reader characters = input.getCharacterStream();
        if (characters != null) {
            parse(characters);
        } else {
            InputStream bytes = input.getByteStream();
            if (bytes != null) {
                String encoding = input.getEncoding();
                if (encoding != null) {
                    parse(new InputStreamReader(bytes, encoding));
                } else {
                    parse(bytes);
                }
            } else {
                String systemId = input.getSystemId();
                parse(systemId);
            }
        }
    }

    public void parse(String systemId) throws IOException, SAXException {
        parse(new File(systemId));
    }

    public void parse(File file) throws IOException, SAXException {
        parse(new FileInputStream(file));
    }

    public abstract void parse(InputStream in) throws IOException, SAXException;

    public abstract void parse(Reader reader) throws IOException, SAXException;
}
