package net.jhorstmann.jsonp.xmlutil;

import org.xml.sax.Attributes;
import org.xml.sax.ContentHandler;
import org.xml.sax.SAXException;
import org.xml.sax.XMLReader;

public class NestedContentHandler extends DefaultContentHandler {

    protected final XMLReader reader;
    protected final NestedContentHandler parent;
    private final String namespace;
    private int depth;

    public NestedContentHandler(XMLReader reader) {
        this(reader, null);
    }

    public NestedContentHandler(XMLReader reader, NestedContentHandler parent) {
        this(reader, parent, "");
    }

    public NestedContentHandler(XMLReader reader, NestedContentHandler parent, String namespace) {
        this.reader = reader;
        this.parent = parent;
        this.depth = 0;
        this.namespace = namespace != null ? namespace : "";
    }

    public final void pushContentHandler(ContentHandler handler) {
        reader.setContentHandler(handler);
    }

    public final void popContentHandler() {
        reader.setContentHandler(parent);
    }

    public void startNestedElement(String localName, Attributes atts) throws SAXException {
    }

    public void startNestedElement(String localName, Attributes atts, int depth) throws SAXException {
        if (depth == 0) {
            startNestedElement(localName, atts);
        }
    }

    public void startNestedElement(String uri, String localName, String qName, Attributes atts, int depth) throws SAXException {
        if (namespace.equals(uri)) {
            startNestedElement(localName, atts, depth);
        }
    }

    public void endNestedElement(String localName, int depth) throws SAXException {
    }

    public void endNestedElement(String uri, String localName, String qName, int depth) throws SAXException {
        if (namespace.equals(uri)) {
            endNestedElement(localName, depth);
        }
    }

    public void endRootElement(String localName) throws SAXException {
    }

    public void endRootElement(String uri, String localName, String qName) throws SAXException {
        if (namespace.equals(uri)) {
            endRootElement(localName);
        }
    }

    @Override
    public final void startElement(String uri, String localName, String qName, Attributes atts) throws SAXException {
        System.out.println("start : " + localName);
        startNestedElement(uri, localName, qName, atts, depth);
        depth++;
    }

    @Override
    public final void endElement(String uri, String localName, String qName) throws SAXException {
        if (depth == 0) {
            System.out.println("end root : " + localName + " - parent : " + (parent == null ? null : parent.getClass().getSimpleName()));
            endRootElement(uri, localName, qName);
            // The corresponding start tag was seen by the parent handler before it pushed this handler on the stack
            // so we need to correct the parent's depth
            parent.depth--;
            reader.setContentHandler(parent);
        } else {
            depth--;
            System.out.println("end nested (" + depth + "): " + localName + " - parent : " + (parent == null ? null : parent.getClass().getSimpleName()));
            endNestedElement(uri, localName, qName, depth);
        }
    }
}
